#ifndef __CLOTH_HXX
#define __CLOTH_HXX
/*----------------------------------------------------------------------------------------------------*/

#include <Inventor/SbLinear.h>

class Cloth {
public:
  static const float k;
  static float dt;  
  static const float gamma;
    
protected:
  const size_t xMax, yMax;
  const size_t cloth_size;

private:
  SbVec3f *next_vertices;
  SbVec3f *vertices;

  SbVec3f *next_velocities;
  SbVec3f *velocities;

  bool *fixed;

  SbVec3f *ext_forces;
protected:
  void apply_force(size_t idx, const SbVec3f &force);

public:
  /* sequential access to mass points */
  const SbVec3f& operator[](size_t idx) const;

  size_t x_max(void) const;
  size_t y_max(void) const;
  size_t n(void) const;

public:
  /* interface needed by ode-integrator */
  SbVec3f compute_force(const SbVec3f vertices[], const SbVec3f velocities[], size_t idx) const;

  /* integrate next step vertex-positions */  
  void update(void);

  /* fix a masspoint to a certain coordinate */
  void fix_masspoint(size_t x, size_t y, const SbVec3f &pos);
  bool is_fixed(size_t idx) const;  
  const SbVec3f& get_masspoint(size_t x, size_t y) const;
  
  Cloth(void);
};

extern inline const SbVec3f& Cloth::operator[](size_t idx) const {
  return vertices[idx];
}

extern inline const SbVec3f& Cloth::get_masspoint(size_t x, size_t y) const {
  return (*this)[y*xMax+x];
}

extern inline bool Cloth::is_fixed(size_t idx) const {
  return fixed[idx];
}

extern inline size_t Cloth::x_max() const {
  return xMax;
}

extern inline size_t Cloth::y_max() const {
  return yMax;
}

extern inline size_t Cloth::n() const {
  return cloth_size;
}

extern inline void Cloth::apply_force(size_t idx, const SbVec3f &force) {
  ext_forces[idx] += force;
}
/*----------------------------------------------------------------------------------------------------*/
#endif
