#include <Inventor/sensors/SoIdleSensor.h> 
#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include "cloth_visualizer.hxx"
#include <iostream>
#include <cstdio>

static SoQtExaminerViewer *eviewer = NULL;
static SoSeparator *root = NULL;
static VisualCloth *cloth = NULL;
static SoText2 *dt_text = NULL;

//Idle function
static void idlefunc(void *data, SoSensor* s) {
  static struct timespec to = {0, (long int)1e7};
  char dt_str[1000];
  
  std::sprintf(dt_str,"current timestep: %f s", cloth->dt);  
  dt_text -> string.setValue(dt_str);
  to.tv_nsec = (long int)(cloth->dt*1e9);
  assert(to.tv_nsec > 0);
  cloth->update();  
  cloth->draw();
  s->schedule();
  nanosleep(&to, 0);
}

/* used for stepwise simulation of texile movement */
static void keyboardCallback(void *dt, SoEventCallback *eCB) {
  //SoKeyboardEvent kbEvent = eCB -> getEvent();
  cloth -> update();
  std::cout << "kb event\n";
}

int main(int argc, char *argv[])
{	
  QWidget *mainwin = SoQt::init(argc, argv, argv[0]);

  root = new SoSeparator;
  SoBaseColor *col = new SoBaseColor;
  SoEventCallback *evenCB = new SoEventCallback;
  cloth = new VisualCloth;

  {
    SoSeparator *txt_sep = new SoSeparator;
    SoTranslation *trns = new SoTranslation;
    SoBaseColor *txt_col = new SoBaseColor;

    trns -> translation.setValue(-3, 2, 0);
    txt_col -> rgb = SbColor(1,1,1);
    dt_text = new SoText2;
    txt_sep -> addChild(dt_text);
    txt_sep -> addChild(trns);
    txt_sep -> addChild(txt_col);    
    root -> addChild(txt_sep);
  }

  // Create idle sensor and pass pointer to scene
  SoIdleSensor *idle = new SoIdleSensor(idlefunc, NULL);

  root -> ref();

  col -> rgb = SbColor(1,1,0);  
  evenCB -> addEventCallback(SoKeyboardEvent::getClassTypeId(), keyboardCallback, NULL);
  evenCB -> setPath(NULL);

  root -> addChild(col);
  root -> addChild(evenCB);
  cloth -> fix_masspoint(0,cloth->y_max()-1,(*cloth)(0,cloth->y_max()-1));
  cloth -> fix_masspoint(cloth->x_max()-1,cloth->y_max()-1,(*cloth)(cloth->x_max()-1,cloth->y_max()-1));
  cloth -> fix_masspoint(0,0,(*cloth)(0,0)-SbVec3f(-0.4,-0.3,0.5));
  root -> addChild(cloth -> generate());
  
  
  // Use one of the convenient SoQt viewer classes.
  eviewer = new SoQtExaminerViewer(mainwin);
  eviewer->setSceneGraph(root);
  eviewer->show();
  
  idle->schedule();

  // Pop up the main window.
  SoQt::show(mainwin);
  // Loop until exit.
  SoQt::mainLoop();
  
  // Clean up resources.
  delete eviewer;
  root->unref();  

  return 0;
}
