#include <cassert>
#include <Inventor/SbBox.h>
#include <iostream>
using namespace std;
#include "colliding_cloth.hxx"

bool CollidingCloth::do_collide(const CollidingCloth::ColliderTriangle &c, 
				const CollidingCloth::ColliderTriangle &test, 
				int idx) {
  /* test wheter vector between mass center and corner point of test,
     crosses the plane spanned by c */
  SbVec3f b = (*this)[c.indexes[0]] - test.center;
  SbVec3f prm;
  SbMatrix mat(test.c_rel[idx][0], -c.side[0][0], -c.side[1][0], 0,
	       test.c_rel[idx][1], -c.side[0][1], -c.side[1][1], 0,
	       test.c_rel[idx][2], -c.side[0][2], -c.side[1][2], 0,
	       0,                  0,             0,             1);
    
  if (mat.det3() == 0) return false;
  mat.inverse().multMatrixVec(b, prm);

  if (prm[0] > 0 && prm[0] < 1 &&
      prm[1] > 0 && prm[1] < 1 &&
      prm[2] > 0 && prm[2] < 1) return true;
  else return false;    
}

static const float force_coeff = 1e-2;

void CollidingCloth::update() {
  Cloth::update();
  recompute_trias();

  /* resolve self-collision */
  for (size_t t = 0; t < nof_trias; t++) {
    for (size_t tt = 0; tt < nof_trias; tt++) {
      if (tt != t) {
	for (int cs = 0; cs < 3; cs++) {
	  if (do_collide(trias[t], trias[tt], cs)) {
	    SbVec3f n;

	    n = trias[t].side[0].cross(trias[t].side[1]);
	    n.normalize();
	    apply_force(trias[tt].indexes[cs], force_coeff*Cloth::k*n);
	    /* actio == reactio */
	    apply_force(trias[t].indexes[0], -force_coeff*Cloth::k*n/3);
	    apply_force(trias[t].indexes[1], -force_coeff*Cloth::k*n/3);
	    apply_force(trias[t].indexes[2], -force_coeff*Cloth::k*n/3);
	  }
	}
      } /* if (tt != t) */
    }
  } /* for (t) */
}

/*
  Generate collider Triangles
*/
void CollidingCloth::init() {
  trias = new ColliderTriangle[2*(x_max()-1)*(y_max()-1)];
  nof_trias = 0;

  for (size_t v = 0; v < n(); v++) {
    if (v%x_max() != x_max()-1 && v/x_max() != y_max()-1) {
      trias[nof_trias].indexes[0] = v;
      trias[nof_trias].indexes[1] = v+1;
      trias[nof_trias].indexes[2] = v+x_max();     
      nof_trias++;
    }

    if (v%x_max() != 0 && v/x_max() != 0) {
      trias[nof_trias].indexes[0] = v;
      trias[nof_trias].indexes[1] = v-1;
      trias[nof_trias].indexes[2] = v-x_max();     
      nof_trias++;
    }
  }

  assert(nof_trias == 2*(x_max()-1)*(y_max()-1));
  recompute_trias();
}
 
void CollidingCloth::recompute_trias() {
   for (size_t i = 0; i < nof_trias; i++) {
     { /* compute mass center */
       SbVec3f side[2], b;
       float p;

       side[0] = (float)0.5*((*this)[trias[i].indexes[0]] + (*this)[trias[i].indexes[1]]);
       side[0] -= (*this)[trias[i].indexes[2]];

       side[1] = (float)0.5*((*this)[trias[i].indexes[0]] + (*this)[trias[i].indexes[2]]);
       side[1] -= (*this)[trias[i].indexes[1]];

       b = (*this)[trias[i].indexes[1]] - (*this)[trias[i].indexes[2]];
       
       /* solve 2x2 equation system in order to find the coefficients for the 
	  vectors "side" describing the triangle center */
       p = (b[1]*(side[0][0]/side[0][1]) - b[0])/(-side[0][0]/side[0][1]*side[1][1] + side[1][0]);
       assert(p <= 1 && p >= 0);
       trias[i].center = p*side[1] + (*this)[trias[i].indexes[1]];
     }

     trias[i].side[0] = (*this)[trias[i].indexes[1]] - (*this)[trias[i].indexes[0]];
     trias[i].side[1] = (*this)[trias[i].indexes[2]] - (*this)[trias[i].indexes[0]];

     for (int v = 0; v < 3; v++) trias[i].c_rel[v] = (*this)[trias[i].indexes[v]] - trias[i].center;
   }
}

CollidingCloth::CollidingCloth() : Cloth() {
  init();
}
